﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.IntegerVectorEncoding;
using HeuristicLab.Operators;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.GeneralizedQuadraticAssignment {
  [Item("NMoveMaker", "Performs an n-move.")]
  [StorableClass]
  public class NMoveMaker : SingleSuccessorOperator, IGQAPNMoveOperator {

    public ILookupParameter<IntegerVector> AssignmentParameter {
      get { return (ILookupParameter<IntegerVector>)Parameters["Assignment"]; }
    }
    public ILookupParameter<NMove> MoveParameter {
      get { return (ILookupParameter<NMove>)Parameters["Move"]; }
    }
    public ILookupParameter<DoubleValue> QualityParameter {
      get { return (ILookupParameter<DoubleValue>)Parameters["Quality"]; }
    }
    public ILookupParameter<DoubleValue> InfeasibilityParameter {
      get { return (ILookupParameter<DoubleValue>)Parameters["Infeasibility"]; }
    }
    public ILookupParameter<DoubleValue> MoveQualityParameter {
      get { return (ILookupParameter<DoubleValue>)Parameters["MoveQuality"]; }
    }
    public ILookupParameter<DoubleValue> MoveInfeasibilityParameter {
      get { return (ILookupParameter<DoubleValue>)Parameters["MoveInfeasibility"]; }
    }

    [StorableConstructor]
    protected NMoveMaker(bool deserializing) : base(deserializing) { }
    protected NMoveMaker(NMoveMaker original, Cloner cloner) : base(original, cloner) { }
    public NMoveMaker()
      : base() {
      Parameters.Add(new LookupParameter<IntegerVector>("Assignment", "The equipment-location assignment vector."));
      Parameters.Add(new LookupParameter<NMove>("Move", "The move to perform."));
      Parameters.Add(new LookupParameter<DoubleValue>("Quality", "The solution quality."));
      Parameters.Add(new LookupParameter<DoubleValue>("Infeasibility", "The infeasibility of a solution."));
      Parameters.Add(new LookupParameter<DoubleValue>("MoveQuality", "The quality of the move if it would be applied."));
      Parameters.Add(new LookupParameter<DoubleValue>("MoveInfeasibility", "The infeasibility of the move if it would be applied."));
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new NMoveMaker(this, cloner);
    }

    public static void Apply(IntegerVector vector, NMove move) {
      for (int i = 0; i < move.N; i++) {
        vector[move.Equipments[i]] = move.Locations[i];
      }
    }

    public override IOperation Apply() {
      Apply(AssignmentParameter.ActualValue, MoveParameter.ActualValue);
      QualityParameter.ActualValue.Value = MoveQualityParameter.ActualValue.Value;
      InfeasibilityParameter.ActualValue.Value = MoveInfeasibilityParameter.ActualValue.Value;
      return base.Apply();
    }
  }
}
